﻿using BlazorTest.Components;
using Microsoft.AspNetCore.Components;
using Syncfusion.EJ2.Blazor.Navigations;
using System;
using System.Collections.Generic;
using System.Linq;

namespace BlazorTest
{
    public class IndexModel : LayoutComponentBase
    {
        public IndexModel()
        {
            TestFragment = Builder();
        }

        protected EjsSidebar sidebar;
        protected Dictionary<string, object> htmlAttribute = new Dictionary<string, object>()
        {
            {"class", "sidebar-menu" }
        };
        public List<MenuItem> AccountMenuItems = new List<MenuItem> {
            new MenuItem {
                Text = "Account",
                Items = new List<MenuItem> {
                    new MenuItem { Text = "Profile" },
                    new MenuItem { Text = "Sign out" }
                }
            }
        };

        protected EjsTab tabObject;

        protected void onSelect(MenuEventArgs args)
        {
            try
            {
                var tabItemClass = new TabItemClass
                {
                    Id = Guid.NewGuid(),
                    HeaderText = args.Item.Text
                };
                Console.WriteLine(tabItemClass.Id);
                var menuItem = FindById(MainMenuItems, args.Item.Id);
                if (menuItem is MyMenuItem myMenuItem)
                {
                    tabItemClass.SetComponentType(myMenuItem.ComponentType);
                }
                TabItems.Add(tabItemClass);

                StateHasChanged();
            }
            catch (Exception ex)
            {
                Console.WriteLine(ex);
            }
        }

        private MenuItem FindById(List<MenuItem> menuItems, string id)
        {
            foreach (var menuItem in menuItems)
            {
                if (menuItem.Id == id)
                {
                    return menuItem;
                }

                if (menuItem.Items?.Any() ?? false)
                {
                    var subMenuItem = FindById(menuItem.Items, id);
                    if (subMenuItem != null) return subMenuItem;
                }
            }
            return null;
        }

        public RenderFragment TestFragment { get; set; }

        private RenderFragment Builder() => builder =>
        {
            Console.WriteLine("Building content");

            builder.OpenElement(0, "div");
            builder.OpenComponent(1, typeof(Test1));
            builder.CloseComponent();
            builder.CloseElement();

            base.BuildRenderTree(builder);
        };

        public List<MenuItem> MainMenuItems = new List<MenuItem>{
            new MyMenuItem("Test1", typeof(Test1)) {
                IconCss = "icon-user icon"
            },
            new MyMenuItem("Test2", typeof(Test2)) {
                IconCss = "icon-bell-alt icon"
            }
        };

        public void Toggle()
        {
            sidebar.Toggle();
        }

        public List<TabItemClass> TabItems { get; set; } = new List<TabItemClass>();

        public void RemoveTabItem(TabItemClass tabItem)
        {
            if (tabItem != null)
            {
                Console.WriteLine($"Removing {tabItem.Id}");
                TabItems = TabItems.Where(o => o.Id != tabItem.Id).ToList();
                StateHasChanged();
            }
        }
    }

    public class TabItemClass
    {
        public TabItemClass()
        {
            this.ContentBuilder = builder =>
            {
                if (_componentType != null)
                {
                    builder.OpenComponent(0, _componentType);
                    builder.CloseComponent();
                }
            };
        }

        public Guid Id { get; set; }

        private Type _componentType;

        public string HeaderText { get; set; }

        public void SetComponentType(Type type)
        {
            if (typeof(IComponent).IsAssignableFrom(type))
                _componentType = type;
        }

        public RenderFragment ContentBuilder;
    }
}

namespace Syncfusion.EJ2.Blazor.Navigations
{
    public class MyMenuItem : MenuItem
    {
        public MyMenuItem(string text, Type componentType)
        {
            Text = text;
            ComponentType = componentType;
            Id = componentType.FullName;
        }

        public Type ComponentType { get; set; }
    }
}
